#include "Element.mqh"
Element createTextArea( 
   string  name,
   string  parent,
   Element &array[],
   int     x,
   int     y,
   string  text,
   int     columns         = 63,
   int     padding         = 20,
   int     lineSpacing     = 4,
   int     borderWidth     = 1,
   color   textColor       = clrBlack,
   color   backgroundColor = clrWhite,
   color   borderColor     = clrLightGray
)
{
   
   Element textArea    = defaultElement;
   textArea.parent     = parent;
   textArea.name       = parent + "TextArea" + name;
   textArea.x          = x;
   textArea.y          = y;
   textArea.clr        = borderColor;
   textArea.bg_clr     = backgroundColor;
   textArea.line_width = borderWidth;
   textArea            = createElement( textArea, 39 );
   addElement( textArea, array );
   
   Element textLine    = defaultElement;
   textLine.parent     = textArea.name; 
   
   if( columns > 63 || columns < 1 )columns = 63;
    
   
   ushort arr[];
   string line; 
   string remainder;
   int    h            = 0;
   int    w            = 0;
   int    lineId       = 0;
   int    arrStart     = 0;
   int    wOffset      = 0;
   int    maxChar      = columns;
   int    lineY        = textArea.y + padding;
   
   StringToShortArray( text, arr );
   int    charTotal    = ArraySize(arr);
   
   if( charTotal < maxChar ){
          maxChar      = charTotal;
   }
   
   int    lines        = charTotal/maxChar;
   
   for( int i = 0; i < lines; i++ )
   {
      lineId++;
      line             = ShortArrayToString( arr, arrStart, maxChar );
      textLine.name    = textArea.name + "TextLine" + (string)( lineId );
      textLine.text    = line;
      textLine.x       = textArea.x + padding;
      textLine.y       = lineY;
      textLine.clr     = textColor;
      createElement( textLine, 33 );
      addElement( textLine, array );
      
      arrStart         = arrStart + maxChar;
      int lineW        = 0;
      TextGetSize( line, lineW, h );
      if( lineW > w )w = lineW;
      h                = h + lineSpacing;
      lineY            = lineY + h; 
   }
   
   if( arrStart < charTotal ){
      
      lineId++;
      remainder        = ShortArrayToString( arr, arrStart );
      textLine.name    = textArea.name + "TextLine" + (string)( lineId );
      textLine.text    = remainder;
      textLine.y       = lineY;
      createElement( textLine, 33 );
      addElement( textLine, array );
      
   }
   if( columns > 18 ){
      wOffset = padding;
      if( padding < 20 )wOffset = 22;
   }
   textArea.height     = ((lineId) * h) + ( padding * 2 ) - lineSpacing;
   textArea.width      = w + wOffset +( padding * 2 );
   modifyElement( textArea, array );
   
   string fileName     =  parent + "Elements.BMP";
   int imageWidth      = (int)( ChartGetInteger( textArea.chart_ID, CHART_WIDTH_IN_PIXELS ) );
   int imageHeight     = (int)( ChartGetInteger( textArea.chart_ID, CHART_HEIGHT_IN_PIXELS ) );
   ChartScreenShot( textArea.chart_ID, fileName, imageWidth, imageHeight, ALIGN_RIGHT );
   ObjectsDeleteAll( textArea.chart_ID, textArea.name );
   
   Element textImage   = defaultElement;
   textImage.name      = textArea.name;
   textImage.text      = "Text Area";
   textImage.x         = x;
   textImage.y         = y;
   textImage.x_offset  = textArea.x + 3;
   textImage.y_offset  = textArea.y + 3;
   textImage.width     = textArea.width;
   textImage.height    = textArea.height;
   textImage.file      = "\\Files\\" + fileName;
   textImage.file_off  = "\\Files\\" + fileName;
   textImage.file_on   = "\\Files\\" + fileName;
   textImage           = createElement( textImage, 36 ); 
   addElement( textImage, array );
   
   //Print(h);
   return textArea;
}